import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;
import javax.swing.event.*;


class CommandPanel extends JPanel implements ActionListener, ChangeListener
{
    private Simulation simu;
    private JButton    playButton, pauseButton, stepByStepButton;
    private JCheckBox  wrapCheckBox, graphicInfoCheckBox, preyGroupingCheckBox, predatorGroupingCheckBox;
    private JSpinner   preyVisionSpinner, predatorVisionSpinner, preyGroupingNb, predatorGroupingNb;
    private JSlider    preySpeedSlider, simulationSpeedSlider;
    private JLabel     preySpeedLabel, simulationSpeedLabel, preyStrategiesLabel, predatorStrategiesLabel;
    private JComboBox  preyStrategiesComboBox, predatorStrategiesComboBox;
    
    private JPanel createSimulationManagementPanel()
    {
	JPanel p = new JPanel();

	/** A border around the panel */
	TitledBorder title;
	title = BorderFactory.createTitledBorder("Gestion de la simulation");
	p.setBorder(title);

	/* Create layout and constraints */
	GridBagLayout gridbag = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	p.setLayout(gridbag);

	c.fill    = GridBagConstraints.HORIZONTAL;
	c.weightx = 0.5;

	playButton       = new JButton("Dmarrer");
	pauseButton      = new JButton("Pause");
	stepByStepButton = new JButton("Pas  pas");

	playButton.addActionListener(this);
	pauseButton.addActionListener(this);
	stepByStepButton.addActionListener(this);

	pauseButton.setEnabled(false);

	c.gridy = 0;

	c.gridx = 0;
	p.add(playButton, c);

	c.gridx = 1;
	p.add(pauseButton, c);

	c.gridy++;
	c.gridwidth = 2;
	c.gridx = 0;
	p.add(stepByStepButton, c);

        simulationSpeedLabel  = new JLabel("Vitesse de simulation : 100%", JLabel.CENTER);
        simulationSpeedLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
	simulationSpeedSlider = new JSlider(JSlider.HORIZONTAL,
					      1, 100 , 100);
	simulationSpeedSlider.addChangeListener(this);
	simulationSpeedSlider.setMajorTickSpacing(500);
	simulationSpeedSlider.setPaintTicks(true);
	simulationSpeedSlider.setPaintLabels(false);
	simulationSpeedSlider.setBorder(BorderFactory.createEmptyBorder(0,0,15,0));

	c.gridy ++;
	c.gridx = 0;
	c.gridwidth = 2;	
	p.add(simulationSpeedLabel, c);

	c.gridy ++;
	c.gridx = 0;
	c.gridwidth = 2;
	p.add(simulationSpeedSlider, c);

	return p;
    }

    private JPanel createAnimalsManagementPanel(int sizeX, int sizeY,
						int nbPrey, int nbPredator)
    {
	int smallestDimension = (sizeX < sizeY) ? sizeX : sizeY;

	JPanel p = new JPanel();

	/** A border around the panel */
	TitledBorder title;
	title = BorderFactory.createTitledBorder("Gestion des animaux");
	p.setBorder(title);

	/** Create layout and constraints */
	GridBagLayout gridbag = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	p.setLayout(gridbag);

	c.fill    = GridBagConstraints.HORIZONTAL;
	c.weightx = 0.0;
	
	/** Label + spinner for the vision of preys */
	preyVisionSpinner = 
	    new JSpinner(new SpinnerNumberModel(Prey.DEFAULT_VISION, 
						1, smallestDimension, 1));

	preyVisionSpinner.addChangeListener(this);

	JLabel preyVisionLabel = new JLabel("Vision des proies", 
					    JLabel.LEFT);

	c.gridy = 0;

	c.gridx = 0;
	c.gridwidth = 2;
	p.add(preyVisionLabel, c);

	c.gridx = 2;
	c.gridwidth = 1;
	p.add(preyVisionSpinner, c);

	/* Label + spinner for the vision of predators */
	predatorVisionSpinner = 
	    new JSpinner(new SpinnerNumberModel(Predator.DEFAULT_VISION, 
						1, smallestDimension, 1));

	predatorVisionSpinner.addChangeListener(this);

	JLabel predatorVisionLabel = new JLabel("Vision prdateurs", 
						JLabel.LEFT);


	c.gridy ++;
	
	c.gridx = 0;
	c.gridwidth = 2;
	p.add(predatorVisionLabel, c);

	c.gridx = 2;
	c.gridwidth = 1;
	p.add(predatorVisionSpinner, c);

	/** Label + spinner for the number of preys per group */
	preyGroupingNb = 
	    new JSpinner(new SpinnerNumberModel(Prey.DEFAULT_GROUPING_NB, 1, nbPrey+100, 1));

	preyGroupingNb.addChangeListener(this);
	preyGroupingNb.setEnabled(Prey.DEFAULT_GROUPING);

	JLabel preyGroupingNbLabel = new JLabel("Nb proies/groupe ", 
						JLabel.LEFT);

	c.gridy++;

	c.gridx = 0;
	c.gridwidth = 2;
	p.add(preyGroupingNbLabel, c);

	c.gridx = 2;
	c.gridwidth = 1;
	p.add(preyGroupingNb, c);

	/** Label + spinner for the number of predators per group */
	predatorGroupingNb = 
	    new JSpinner(new SpinnerNumberModel(Predator.DEFAULT_GROUPING_NB, 
						1, nbPredator+100, 1));
	
	predatorGroupingNb.addChangeListener(this);
	predatorGroupingNb.setEnabled(Predator.DEFAULT_GROUPING);

	JLabel predatorGroupingNbLabel = new JLabel("Nb prdateurs/groupe ", 
						    JLabel.LEFT);
	
	c.gridy++;
	
	c.gridx = 0;
	c.gridwidth = 2;
	p.add(predatorGroupingNbLabel, c);

	c.gridx = 2;
	c.gridwidth = 1;
	p.add(predatorGroupingNb, c);	

	preyGroupingCheckBox = new JCheckBox("Regroupement des proies", Prey.DEFAULT_GROUPING);
	preyGroupingCheckBox.addActionListener(this);

	c.gridy++;	
	c.gridwidth = 3;
	c.gridx = 0;
	p.add(preyGroupingCheckBox, c);

	predatorGroupingCheckBox = new JCheckBox("Regroupement des prdateurs", Predator.DEFAULT_GROUPING);
	predatorGroupingCheckBox.addActionListener(this);	

	c.gridy++;
	c.gridwidth = 3;
	c.gridx = 0;

	p.add(predatorGroupingCheckBox, c);

	/* The speed slider */
        preySpeedLabel = new JLabel("Vitesse proies/prdateurs : " + Prey.DEFAULT_SPEED + " %", 
				      JLabel.CENTER);
	preySpeedSlider = new JSlider(JSlider.HORIZONTAL,
				      0, 100 , Prey.DEFAULT_SPEED);
	preySpeedSlider.addChangeListener(this);
	preySpeedSlider.setMajorTickSpacing(500);
	preySpeedSlider.setPaintTicks(true);
	preySpeedSlider.setPaintLabels(false);

	c.gridy ++;
	c.gridx = 0;
	c.gridwidth = 3;
	p.add(preySpeedLabel, c);

	c.gridy ++;
	c.gridx = 0;
	c.gridwidth = 3;
	p.add(preySpeedSlider, c);

	/* The prey's strategy combo box */
	JLabel preyStrategiesLabel = new JLabel("Proie : ", 
						    JLabel.LEFT);
	String[] preyStrategies = {"Fabien Michel", "fuite"};
	preyStrategiesComboBox = new JComboBox(preyStrategies);
	preyStrategiesComboBox.setSelectedIndex(0);
	preyStrategiesComboBox.addActionListener(this);
	
	c.gridy++;
	
	c.gridx = 0;
	c.gridwidth = 1;
	p.add(preyStrategiesLabel, c);

	c.gridx = 1;
	c.gridwidth = 2;
	p.add(preyStrategiesComboBox, c);	

	
	/* The predator's strategy combo box */
	JLabel predatorStrategiesLabel = new JLabel("Prd. : ", 
						    JLabel.LEFT);
	String[] predatorStrategies = {"Fabien Michel", 
					"attirance", 
					"timeout", 
					"timeout et dir. alatoire"};
	predatorStrategiesComboBox = new JComboBox(predatorStrategies);
	predatorStrategiesComboBox.setSelectedIndex(0);
	predatorStrategiesComboBox.addActionListener(this);
	
	c.gridy++;
	
	c.gridx = 0;
	c.gridwidth = 1;
	p.add(predatorStrategiesLabel, c);

	c.gridx = 1;
	c.gridwidth = 2;
	p.add(predatorStrategiesComboBox, c);	
	return p;
    }

    /**
     * Create the panel with the simulation options (wrap/nowrap,
     * graphical informations, etc...)
     *
     * @result The created panel
     */
    private JPanel createSimulationOptionPanel()
    {
	JPanel p = new JPanel();

	/** A border around the panel */
	TitledBorder title;
	title = BorderFactory.createTitledBorder("Options de la simulation");
	p.setBorder(title);

	/** Create layout and constraints */
	GridBagLayout gridbag = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	p.setLayout(gridbag);

	c.fill    = GridBagConstraints.HORIZONTAL;
	c.weightx = 0.5;
	
	wrapCheckBox = new JCheckBox("Monde torique", true);
	wrapCheckBox.addActionListener(this);

	c.gridy = 0;

	c.gridx = 0;
	p.add(wrapCheckBox, c);
	

	c.gridy ++;
	graphicInfoCheckBox = new JCheckBox("Informations graphiques", false);
	graphicInfoCheckBox.addActionListener(this);

	c.gridx = 0;

	p.add(graphicInfoCheckBox, c);
	
	return p;
    }

    /**
     * Create the command panel.
     *
     * @param s The current simulation beeing managed
     */
    public CommandPanel(Simulation s, int sizeX, int sizeY, int nbPrey, int nbPredator)
    {
	simu = s;

	/** Create layout and constraints */
	GridBagLayout gridbag = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	setLayout(gridbag);

	c.fill = GridBagConstraints.BOTH;
	c.weightx = 0.5;

	JPanel simulationManagementPanel = createSimulationManagementPanel();
	
	JPanel animalManagementPanel     = createAnimalsManagementPanel(sizeX, 
									sizeY,
									nbPrey,
									nbPredator);

	JPanel simulationOptionPanel     = createSimulationOptionPanel();

	c.gridy = 0;
	c.gridx = 0;
	add(simulationManagementPanel, c);

	c.gridy = 1;
	add(simulationOptionPanel, c);	

	c.gridy = 2;
	add(animalManagementPanel, c);	

	validate();
    }

    public void actionPerformed(ActionEvent e)
    {
	Object s=e.getSource();
	
	if(s == playButton)
	    {
		simu.setSchedulerMode(TurtleScheduler.RUNNING_MODE);
		pauseButton.setEnabled(true);
		playButton.setEnabled(false);
	    }
	else if(s == pauseButton)
	    {
		simu.setSchedulerMode(TurtleScheduler.PAUSED_MODE);
		pauseButton.setEnabled(false);
		playButton.setEnabled(true);
	    }
	else if(s == stepByStepButton)
	    {
		pauseButton.setEnabled(false);
		playButton.setEnabled(true);
		simu.setSchedulerMode(TurtleScheduler.STEP_BY_STEP_MODE);
		simu.schedulerGiveOneMoreStep();
	    }
	else if(s == wrapCheckBox)
	    {
		int oldMode = simu.getSchedulerMode();

		simu.setSchedulerMode(TurtleScheduler.PAUSED_MODE);

		simu.setWrappingMode(wrapCheckBox.isSelected());

		simu.setSchedulerMode(oldMode);
	    }
	else if(s == graphicInfoCheckBox)
	    {
		simu.setGraphicInfoMode(graphicInfoCheckBox.isSelected());
	    }
	else if(s == preyGroupingCheckBox)
	    {
		preyGroupingNb.setEnabled(preyGroupingCheckBox.isSelected());
		simu.setPreyGrouping(preyGroupingCheckBox.isSelected());
	    }
	else if(s == predatorGroupingCheckBox)
	    {
		predatorGroupingNb.setEnabled(predatorGroupingCheckBox.isSelected());
		simu.setPredatorGrouping(predatorGroupingCheckBox.isSelected());
	    }
    	else if(s == preyStrategiesComboBox)
	    {
		JComboBox comboBox = (JComboBox) s;
		simu.setPreyStrategy(comboBox.getSelectedIndex());
	    }
    	else if(s == predatorStrategiesComboBox)
	    {
		JComboBox comboBox = (JComboBox) s;
		simu.setPredatorStrategy(comboBox.getSelectedIndex());
	    }
    }

    public void stateChanged(ChangeEvent e) 
    {
	Object source = e.getSource();
	
	if(source == preySpeedSlider)
	    {
		JSlider slider = (JSlider) source;

		preySpeedLabel.setText("Vitesse proies/prdateurs : " + 
					 (int) slider.getValue() + " %");
		
		if(!slider.getValueIsAdjusting())
		    {
			simu.setPreySpeed(slider.getValue());
		    }
    }
	if(source == simulationSpeedSlider)
	    {
		JSlider slider = (JSlider) source;

		int val = (int) slider.getValue();

		simulationSpeedLabel.setText("Vitesse de simulation : " 
					     + val + "%");
		
		if (!slider.getValueIsAdjusting()) 
		    {
			simu.setSimulationSpeed(val);
		    }
	    }
	else if(source == preyVisionSpinner)
	    {
		JSpinner spin = (JSpinner) source;

		simu.setPreyVisionRadius(((Integer) 
					  spin.getValue()).intValue());
	    }
	else if(source == predatorVisionSpinner)
	    {
		JSpinner spin = (JSpinner) source;

		simu.setPredatorVisionRadius(((Integer) 
					      spin.getValue()).intValue());
	    }
	else if(source == preyGroupingNb)
	    {
		JSpinner spin = (JSpinner) source;
		
		simu.setPreyGroupingNb(((Integer) 
					spin.getValue()).intValue());
	    }
	else if(source == predatorGroupingNb)
	    {
		JSpinner spin = (JSpinner) source;

		simu.setPredatorGroupingNb(((Integer) 
					    spin.getValue()).intValue());
	    }
    }
}
