/*
 * Projet LO43
 * Julien Miltat -- Julien Rosener -- Thomas Petazzoni
 * Semestre automne 2002-2003
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;
import java.io.*;
import java.text.*;
import com.sun.image.codec.jpeg.*;

public class EnvironnementPanel extends JPanel implements MouseListener
{
    /** The color to draw preys */
    private final static Color preyColor = Color.white;
    
    /** The color to draw predators */
    private final static Color predatorColor = Color.red;

    /** Size of the images that are going to be used for the graphical
     * representation */
    public final static int IMAGE_SIZE = 20;

    /** Width of the world */
    private int sizex;

    /** Height of the world */
    private int sizey;

    /** Size of each cell of the world in the graphical
     * representation */
    private int cellSize;

    /** ?? */
    private BufferedImage offscreen;

    /** ?? */
    private Graphics goffscreen;

    /** The simulation being played */
    private Simulation simu;

    /** The background color */
    private Color backgroundColor;

    /** The line around the panel */
    private Color lineColor;

    /** The images to display preys and predators (if needed) */
    private Image predatorImg, preyImg;

    /** Boolean that tells whether the user wants a graphical
     * representation with images or not */
    private boolean withImages;
    
    EnvironnementPanel(Simulation simu, int cellSize, 
		       boolean withImages)
    {
	super();

	System.out.println("[EnvironnementPanel] cellSize " + cellSize);

	this.withImages = withImages;
	
	// Initialisation des champs
	if(withImages)
	    {
		this.cellSize   = cellSize;
		backgroundColor = Color.white;
		lineColor       = Color.black;

		try 
		    {
			InputStream is1 = getClass().getResourceAsStream("prey-small.jpg");
			preyImg = JPEGCodec.createJPEGDecoder(is1).decodeAsBufferedImage();

			InputStream is2 = getClass().getResourceAsStream("predator-small.jpg");
			predatorImg = JPEGCodec.createJPEGDecoder(is2).decodeAsBufferedImage();
		    } 
		catch(FileNotFoundException e) 
		    {
			System.out.println("Fichier non trouv");
		    }
		catch(IOException e)
		    {
			System.out.println("Erreur d'I/O");
		    }
	    }
	else
	    {
		this.cellSize   = cellSize;
		backgroundColor = Color.black;
		lineColor       = Color.white;
	    }

	this.simu     = simu;
	sizex         = simu.getWorldWidth();
	sizey         = simu.getWorldHeight();
	setSize(sizex*cellSize+2, sizey*cellSize+2);
	setVisible(true);

	
	offscreen      = new BufferedImage(getWidth(), getHeight(), 
					   BufferedImage.TYPE_INT_RGB);
	
	if(offscreen == null)
	    System.out.println("OUUUPPPPS : " + getWidth() + "x" + getHeight());
	
	// Preparation de l'image
	goffscreen = offscreen.getGraphics();
	reset();
	addMouseListener(this);
    }
    
    private int convertCoord(int i)
    {
	return i*cellSize+1;
    }

    private int convertCoordInv(int i)
    {
	return ((i-1)/cellSize);
    }

    public void reset()
    {
	goffscreen.setColor(backgroundColor);
	goffscreen.fillRect(1, 1, getWidth()-2, getHeight()-2);
	goffscreen.setColor(lineColor);
	goffscreen.drawRect(0, 0, getWidth()-1, getHeight()-1);	
    }

    public void update(Graphics g)
    {
	paint(g);
    }
    
    public void paint(Graphics g) 
    {
	super.paint(g);
	g.drawImage(offscreen, 0, 0, this);
    }

    public void paintTurtle(int x, int y, int type)
    {
	Color c;
	Image img;

	if(type == Turtle.PREY_TYPE)
	    {
		c   = preyColor;
		img = preyImg;
	    }
	else
	    {
		c   = predatorColor;
		img = predatorImg;
	    }

	if(withImages)
	    {
		goffscreen.drawImage(img, convertCoord(x), 
				     convertCoord(y), null);
	    }
	else
	    {
		goffscreen.setColor(c);
		goffscreen.fillRect(convertCoord(x), convertCoord(y), 
				    cellSize, cellSize);
	    }
    }

    public void paintInfo(int x, int y, int type,
			  int orientation, int visionRadius, 
			  double xAttrac, double yAttrac,
			  double xRepuls, double yRepuls)
    {
	Color c;
	int posX = convertCoord(x) + (int)(cellSize/2);
	int posY = convertCoord(y) + (int)(cellSize/2);

        /*
	NumberFormat NF = NumberFormat.getInstance();
	NF.setMaximumFractionDigits(2); 
	NF.setMinimumFractionDigits(1); 
	String texte = "("+ NF.format(xAttrac) + 
	               " , " + NF.format(yAttrac) + 
		       ")("+ NF.format(xRepuls) + 
		       " , " + NF.format(yRepuls) + ")" ;
	
	String texte = "  " + (int)xRepuls + "";
	*/
	
	if(type == Turtle.PREY_TYPE)
	    {
		c = preyColor;
	    }
	else
	    {
		c = predatorColor;
	    }

	goffscreen.setColor(c);

	if (orientation != -1)
	    {
		goffscreen.drawLine(posX, posY, 
			   posX + (int)(visionRadius*cellSize*Math.cos((Math.PI*orientation)/180)), 
			   posY + (int)(visionRadius*cellSize*Math.sin((Math.PI*orientation)/180)));

		//goffscreen.drawChars(texte.toCharArray(), 0, texte.length(), posX, posY);
	    }
	
	goffscreen.drawOval(posX-visionRadius*cellSize, 
			    posY-visionRadius*cellSize, 
			    visionRadius*cellSize*2, 
			    visionRadius*cellSize*2);
    }

    public void mouseClicked(MouseEvent e) 
    {
	if(simu.getSchedulerMode() == TurtleScheduler.READY_MODE)
	    {
		int x, y;
		int button;
		
		button = e.getButton();
		
		x = convertCoordInv(e.getX());
		y = convertCoordInv(e.getY());
		
		if(simu.isTurtleAt(x, y) == true)
		    {
			simu.delTurtle(x, y);
		    }
		else
		    {
			if(button == MouseEvent.BUTTON1)
			    {
				if(simu.addTurtle(x, y, Turtle.PREY_TYPE) == false)
				    {
					JOptionPane.showMessageDialog(this, 
								      "Nombre de proies trop important");
					
				    }
			    }
			else if(button == MouseEvent.BUTTON3)
			    {
				if(simu.addTurtle(x, y, Turtle.PREDATOR_TYPE) == false)
				    {
					JOptionPane.showMessageDialog(this, 
								      "Nombre de prdateur trop important");
					
				    }
			    }
			
		    }
	    }
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }
    
}
