/*
 * Projet IA54 / GL53
 * Cyril Coquilleau -- Mlanie Bats -- Julien Rosener -- Thomas Petazzoni
 * Semestre automne 2003-2004
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

public class Interface extends JFrame implements ActionListener
{
    final JMenuItem menuNouveau;
    final JMenuItem menuQuitter;
    final JMenuItem menuSauvegarder;
    final JMenuItem menuOuvrir;
    final JMenuItem menuArreter;
    final JMenuItem menuAboutUs;
    JLabel    bienvenue   = null;
    MainPanel mainPanel   = null;
    EnvironnementPanel ep = null;
    Simulation s;

    Interface()
    {
	JMenuBar  menubar        = new JMenuBar();
	JMenu     menuSimulation = new JMenu("Simulation");
	JMenu     menuAbout      = new JMenu("A propos");

	bienvenue      = new JLabel("Pour creer un monde : Simulation/Nouvelle");

	menuNouveau     = new JMenuItem("Nouvelle");
	menuOuvrir      = new JMenuItem("Ouvrir");
	menuSauvegarder = new JMenuItem("Sauvegarder");
	menuArreter     = new JMenuItem("Arrter");
	menuQuitter     = new JMenuItem("Quitter");

	menuAboutUs     = new JMenuItem("A propos...");

	setJMenuBar(menubar);
	menubar.add(menuSimulation);
	menubar.add(menuAbout);

	menuNouveau.setEnabled(true);
	menuNouveau.addActionListener(this);
	menuSimulation.add(menuNouveau);
	menuArreter.setEnabled(false);
	menuArreter.addActionListener(this);
	menuSimulation.add(menuOuvrir);
	menuOuvrir.addActionListener(this);
	menuSimulation.add(menuSauvegarder);
	menuSauvegarder.addActionListener(this);
	menuSauvegarder.setEnabled(false);
	menuSimulation.add(menuArreter);
	menuSimulation.addSeparator();
	menuQuitter.addActionListener(this);
	menuSimulation.add(menuQuitter);

	menuAboutUs.addActionListener(this);
	menuAbout.add(menuAboutUs);

	getContentPane().add(bienvenue, BorderLayout.NORTH);

	setSize(400, 300);
	setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	setTitle("Projet IA54 / GL53 : Simulation proie/prdateur");
	setResizable(false);
	setVisible(true);
    }

    public void actionPerformed(ActionEvent e)
    {
	JMenuItem source = (JMenuItem) (e.getSource());
	
	if(source == menuNouveau)
	    {
		NouveauMondeDialog diag;
		
		diag = new NouveauMondeDialog(this);
		diag.pack();
		diag.setVisible(true);
	    }
	else if(source == menuOuvrir)
	    {
		JFileChooser fc = new JFileChooser();

		int returnVal = fc.showOpenDialog(this);

		if(returnVal != JFileChooser.APPROVE_OPTION)
		    {
			return;
		    }
		
		File file = fc.getSelectedFile();
		Document d = null;
		
		try
		    {
			DocumentBuilderFactory domFactory =
			    DocumentBuilderFactory.newInstance();
			DocumentBuilder domBuilder =
			    domFactory.newDocumentBuilder();
			d = domBuilder.parse(file);
		    }
		catch(org.xml.sax.SAXException exp)
		    {
			System.err.println("'" + fc.getName() + "' is NOT well-formed.\n" + exp.toString());
		    }
		catch(FactoryConfigurationError exp)
		    {
			System.err.println(exp.toString());
		    }
		catch(ParserConfigurationException exp)
		    {
			System.err.println(exp.toString());
		    }
		catch(Exception exp)
		    {
			System.err.println(exp.toString());
		    }

		NodeList nl = d.getElementsByTagName("world");

		if(nl.getLength() != 1)
		    {
			System.err.println("More than one <world> tag : " + nl.getLength());
		    }

		NamedNodeMap nnm = nl.item(0).getAttributes();

		String xString = nnm.getNamedItem("x").getNodeValue();
		String yString = nnm.getNamedItem("y").getNodeValue();
		String withImagesString = nnm.getNamedItem("with-images").getNodeValue();
		String cellsizeString = nnm.getNamedItem("cellsize").getNodeValue();
		
		int x = (new Integer(xString)).intValue();
		int y = (new Integer(yString)).intValue();
		int cellsize = (new Integer(cellsizeString)).intValue();
		boolean withImages = ((cellsizeString.compareTo("yes") == 0) ? true : false);

		createWorld(x, y, 0, 0, cellsize, withImages, d);
	    }
	else if(source == menuSauvegarder)
	    {
		int oldMode = s.getSchedulerMode();

		System.out.println("oldMOde " + oldMode);

		s.setSchedulerMode(TurtleScheduler.PAUSED_MODE);

		JFileChooser fc = new JFileChooser();

		int returnVal = fc.showOpenDialog(this);

		if(returnVal != JFileChooser.APPROVE_OPTION)
		    {
			s.setSchedulerMode(oldMode);
			return;
		    }
		
		File file = fc.getSelectedFile();

		DocumentBuilder domBuilder = null;

		try
		    {
			DocumentBuilderFactory domFactory =
			    DocumentBuilderFactory.newInstance();
			domBuilder = domFactory.newDocumentBuilder();
		    }
		/*
		catch(org.xml.sax.SAXException exp)
		    {
			System.err.println("'" + fc.getName() + "' is NOT well-formed.\n" + exp.toString());
		    }
		catch(FactoryConfigurationError exp)
		    {
			System.err.println(exp.toString());
		    }
		*/
		catch(ParserConfigurationException exp)
		    {
			System.err.println(exp.toString());
		    }
		/*
		catch(Exception exp)
		    {
			System.err.println(exp.toString());
		    }
		*/


		Document d = domBuilder.newDocument();
		
		Element rootElement = d.createElement("world");

		rootElement.setAttribute("x", (new Integer(s.getWorldWidth())).toString());
		rootElement.setAttribute("y", (new Integer(s.getWorldHeight())).toString());
		rootElement.setAttribute("cellsize", (new Integer(s.getCellSize())).toString());
		rootElement.setAttribute("with-images", (s.hasImages() ? "yes" : "no"));

		d.appendChild(rootElement);
		
		Iterator iter = s.getPreyList().iterator();
		
		while(iter.hasNext())
		    {
			Prey p = (Prey) iter.next();

			Element preyElement = d.createElement("animal");

			preyElement.setAttribute("type", "prey");
			preyElement.setAttribute("x", (new Integer(p.xcor())).toString());
			preyElement.setAttribute("y", (new Integer(p.ycor())).toString());

			rootElement.appendChild(preyElement);
		    }

		iter = s.getPredatorList().iterator();
		
		while(iter.hasNext())
		    {
			Predator p = (Predator) iter.next();

			Element predatorElement = d.createElement("animal");

			predatorElement.setAttribute("type", "predator");
			predatorElement.setAttribute("x", (new Integer(p.xcor())).toString());
			predatorElement.setAttribute("y", (new Integer(p.ycor())).toString());

			rootElement.appendChild(predatorElement);
		    }		

		try
		    {
			
			TransformerFactory tranFactory =
			    TransformerFactory.newInstance();
			Transformer aTransformer = tranFactory.newTransformer();
			Source src = new DOMSource(d);
			Result dest = new StreamResult(file);
			aTransformer.transform(src, dest);
		    }
		catch(Exception exp)
		    {
			System.err.println(exp.toString());
		    }

		s.setSchedulerMode(oldMode);
	    }
	else if(source == menuArreter)
	    {
		s.destroy();
		remove(mainPanel);
		getContentPane().add(bienvenue, BorderLayout.NORTH);
		setSize(300, 300);
		repaint();
		menuNouveau.setEnabled(true);
		menuOuvrir.setEnabled(true);
		menuSauvegarder.setEnabled(false);
		menuArreter.setEnabled(false);
	    }
	else if(source == menuAboutUs)
	    {
		JOptionPane.showMessageDialog(this, 
					      "Projet IA54 / GL53\n\n" + 
					      "Ralisation d'une simulation de type proie/prdateur\n" + 
					      "avec des agents MadKit ractifs sans communication\n" +
					      "Semestre automne 2003-2004\n\n" +
					      "Mlanie Bats <melanie.bats@utbm.fr>\n" +
					      "Cyril Coquilleau <cyril.coquilleau@utbm.fr>\n" +
					      "Thomas Petazzoni <thomas.petazzoni@utbm.fr>\n" +
					      "Julien Rosener <julien.rosener@utbm.fr>");
	    }
	else if(source == menuQuitter)
	    {
		System.exit(0);
	    }
    }

    public void createWorld(int sizeX, int sizeY, int nbPrey, int nbPredator,
			    int cellSize, boolean withImages, Document worldDescriptionXml)
    {
	if(bienvenue != null || mainPanel != null)
	    {
		if(bienvenue != null)
		    {
			remove(bienvenue);
		    }
		else
		    {
			remove(mainPanel);
			mainPanel = null;
		    }

		if(withImages)
		    {
			cellSize = EnvironnementPanel.IMAGE_SIZE;
		    }

		s = new Simulation(sizeX, sizeY, cellSize, nbPrey, nbPredator,
				   withImages, worldDescriptionXml);

		ep = s.getEnvironnementPanel();

		PlotPanel pp = s.getPlotPanel();

		CommandPanel cp = 
		    new CommandPanel(s, sizeX, sizeY, nbPrey, nbPredator);

		mainPanel = new MainPanel(ep, cp, pp, sizeX, sizeY, nbPrey, cellSize);
		getContentPane().add(mainPanel);

		pack();
	    }

	if(sizeY*cellSize < 400)
	    {
		setSize(sizeX*cellSize+300, 650+nbPrey);
	    }
	else
	    {
		setSize(sizeX*cellSize+300, sizeY*cellSize+240+nbPrey);
	    }

	setVisible(true);

	menuNouveau.setEnabled(false);
	menuOuvrir.setEnabled(false);
	menuSauvegarder.setEnabled(true);
	menuArreter.setEnabled(true);

	repaint();
    }

    public static void main(String[] args)
    {
	Interface i = new Interface();
    }
}
