import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import javax.swing.*;
import java.beans.*; //Property change stuff

class NouveauMondeDialog extends JDialog implements PropertyChangeListener, ActionListener
{
    private JOptionPane optionPane;

    final String btnOk       = "OK";
    final String btnAnnuler  = "Annuler";

    JSpinner sizeX         = new JSpinner(new SpinnerNumberModel(200, 10, 
								 1000, 1));
    JSpinner sizeY         = new JSpinner(new SpinnerNumberModel(200, 10, 
								 1000, 1));
    JSpinner nbPrey        = new JSpinner(new SpinnerNumberModel(30, 0, 500, 1));
    JSpinner nbPredator    = new JSpinner(new SpinnerNumberModel(20, 0, 500, 1));
    JSpinner cellSize      = new JSpinner(new SpinnerNumberModel(2, 1, 50, 1));
    JCheckBox withImages   = new JCheckBox("", false);

    final Interface frame;
    
    public NouveauMondeDialog(Interface f)
    {
	super(f, true);
	frame = f;

	final String     text           = "Cration d'une nouvelle simulation";
	final JLabel     labelSixeX     = new JLabel("Taille (X)");
	final JLabel     labelSixeY     = new JLabel("Taille (Y)");
	final JLabel     labNbPrey      = new JLabel("Nombre de proies");
	final JLabel     labNbPredator  = new JLabel("Nombre de prdateurs");
	final JLabel     labCellSize    = new JLabel("Taille des cellules");
	final JLabel     labWithImages  = new JLabel("Reprsentation image");

	final JPanel     panel         = new JPanel();

	withImages.addActionListener(this);

	panel.setLayout(new GridLayout(6, 2, 1, 1));
	panel.add(labelSixeX);
	panel.add(sizeX);
	panel.add(labelSixeY);
	panel.add(sizeY);
	panel.add(labNbPrey);
	panel.add(nbPrey);
	panel.add(labNbPredator);
	panel.add(nbPredator);
	panel.add(labCellSize);
	panel.add(cellSize);
	panel.add(labWithImages);
	panel.add(withImages);

	final String help = "Les proies et prdateurs seront positionnes alatoirement sur le terrain.\n" +
	    "Il est possible d'en ajouter par la suite directement sur le terrain\n" +
	    " l'aide de la souris. Un clic gauche ajoute une proie, un clic droit\n" +
	    "ajoute un prdateur. Un clic sur une case occupe supprime l'animal";
	
	Object[] array   = { text, panel, help };
	
	Object[] options = { btnOk, btnAnnuler };

	optionPane = new JOptionPane(array,
				     JOptionPane.QUESTION_MESSAGE,
				     JOptionPane.YES_NO_OPTION,
				     null,
				     options,
				     options[0]);
	setContentPane(optionPane);
	setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);

	setTitle("Nouvelle simulation");

	optionPane.addPropertyChangeListener(this);
    }


    public void propertyChange(PropertyChangeEvent e) {
	String prop = e.getPropertyName();
	
	if(isVisible()
	   && (e.getSource() == optionPane)
	   && (prop.equals(JOptionPane.VALUE_PROPERTY) ||
	       prop.equals(JOptionPane.INPUT_VALUE_PROPERTY)))
	    {
		Object value = optionPane.getValue();
		
		if (value == JOptionPane.UNINITIALIZED_VALUE) 
		    return;
		
		optionPane.setValue(JOptionPane.UNINITIALIZED_VALUE);
		
		/* Bouton OK */
		if(value.equals(btnOk))
		    {
			try
			    {
				frame.createWorld(((Integer) sizeX.getValue()).intValue(),
						  ((Integer) sizeY.getValue()).intValue(),
						  ((Integer) nbPrey.getValue()).intValue(),
						  ((Integer) nbPredator.getValue()).intValue(),
						  ((Integer) cellSize.getValue()).intValue(),
						  withImages.isSelected(), null);
				setVisible(false);
			    }
			catch(Throwable t)
			    {
				JOptionPane.showMessageDialog(this, 
							      "Taille du monde trop importante : " + t);
			    }
		    }
		/* Bouton Annuler ou fermeture de la boite
		 * de dialogue */
		else
		    {
			setVisible(false);
		    }
	    }
    }

    public void actionPerformed(ActionEvent e)
    {
	Object s=e.getSource();

	if(s == withImages)
	    {
		if(withImages.isSelected())
		    {
			cellSize.setEnabled(false);
		    }
		else
		    {
			cellSize.setEnabled(true);
		    }
	    }
    }
}
