/*
* Patch.java -TurtleKit - A 'star logo' in MadKit
* Copyright (C) 2000-2002 Fabien Michel, Mlanie Bats, Cyril
* Coquilleau, Thomas Petazzoni, Julien Rosener.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

import java.util.*;
import java.awt.Color;

/** 
 * Represents one cell of the world. Allows to store the location of
 * the turtles.
 * 
 *  @author  Fabien Michel, Mlanie Bats, Cyril Coquilleau, Thomas
 *           Petazzoni, Julien Rosener.
 *  @version 1.3 18/12/2003
 */
final public class Patch
{
    /** The list of turtles that are currently located on this
     * patch */
    Collection turtlesHere = new HashSet();

    /**
     * Remove a given turtle from the patch.
     *
     * @param t The turtle to remove
     */
    final void removeAgent(Turtle t)
    {
	turtlesHere.remove(t);
    }
    
    /**
     * Add a given turtle to the patch
     *
     * @param t The turtle to add
     */
    final void addAgent (Turtle t)
    {
	turtlesHere.add(t);
	t.position = this;
    }
    
    /**
     * Get all the turtles of the patch, except the given turtle.
     *
     * @param t The turtle to exclude in the list
     * @return  List of turtles
     */
    final synchronized Turtle[] getOtherTurtles(Turtle t)
    {
	Collection c = new HashSet(turtlesHere);
	c.remove(t);
	return (Turtle[]) c.toArray(new Turtle[0]);
    }

    /**
     * Get the number of turtles on the patch.
     *
     * @return Number of turtles on the pach
     */
    final synchronized int size()
    {
	return turtlesHere.size();
    }
    
    /** Get all the turtles of the patch
     *
     * @return List of turtles
     */
    final public Turtle[] getTurtles()
    {
	return (Turtle[]) turtlesHere.toArray(new Turtle[0]);
    }
}
