/*
* Predator.java -TurtleKit - A 'star logo' in MadKit
* Copyright (C) 2000-2002 Fabien Michel
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

import java.awt.Color;
import java.util.*;
import com.sun.image.codec.jpeg.*;
import java.io.*;
import java.awt.Image;

/** A Predator
  @author Fabien MICHEL
  @version 1.1 17/10/2000 */

public class Predator extends Turtle 
{
    public static final int DEFAULT_VISION = 7;
    public static final int DEFAULT_GROUPING_NB = 6;
    public static final boolean DEFAULT_GROUPING = true;

    //Ajouts de Cyril pour la machine a etats
    public static final int BEHAVIOR_WANDERING=1, BEHAVIOR_GROUPING=2, BEHAVIOR_CHASING=3, BEHAVIOR_FORCE_FLEEING=4;
    public static final int STRATEGIE_FABIEN_MICHEL=0, STRATEGIE_ATTIRANCE_NORMALE=1, 
                            STRATEGIE_ATTIRANCE_TIMEOUT_DIR_FIXE=2, STRATEGIE_ATTIRANCE_TIMEOUT_DIR_ALEATOIRE=3;
    private int behavior;
    private int turnsPlayingBehavior;
    private static int nbTurnsChasingMax = 200;
    private static int nbTurnsGroupingMax = 8;
    private static int nbTurnsFleeingMax = 8;    
    
    
    ArrayList preyList;
    ArrayList predatorList;
    Image img;   

    private int absoluteDirection;
    private static final int absoluteDirs[] = { 0, 45, 90, 135, 180, 225, 270, 315 };

    public Predator(TurtleEnvironment w, int xOrig, int yOrig)
    {
	super(w, xOrig, yOrig, Turtle.PREDATOR_TYPE);
	 
	visionRadius = DEFAULT_VISION;
	regroupement = DEFAULT_GROUPING;
	nbParGroupe  = DEFAULT_GROUPING_NB;

	preyList          = new ArrayList();
	predatorList      = new ArrayList();
        behavior          = BEHAVIOR_WANDERING;
        strategy          = STRATEGIE_FABIEN_MICHEL;
        
	absoluteDirection = absoluteDirs[(int) (Math.random()*8)];
    }
    
    public void activate()
    {
	if(!isGroup("hunt"))
	    {
		createGroup(true, "hunt", null, null);
	    }
	
    	requestRole("hunt", "predator", null);
    	requestRole("hunt", "turtle", null);

	randomHeading();

	if (countTurtlesHere()>0)
	    fd(1);
    } 

    public void end()
    {
	System.out.println("[Predator] End");
	leaveGroup("hunt");
    }
     
    public boolean live()
    {
	move();

	return true;
    }
    

    //Change l'orientation de la tortue courante
    public void changeDirection()
    {
       absoluteDirection = absoluteDirs[(int) (Math.random()*8)];
       setHeading(absoluteDirection);
    }
    
    public void setBehavior(int b)
    {
        //Fixe le comportement et reinitialise le nombre de cycles pour le nouveau comportement a 0
        behavior = b;
        turnsPlayingBehavior = 0;

        //Si l'on est en mode direction aleatoire, on change la direction du predateur
        if (strategy==STRATEGIE_ATTIRANCE_TIMEOUT_DIR_ALEATOIRE) 
            absoluteDirection = absoluteDirs[(int) (Math.random()*8)];
    }
    
    
    private void move()
    {
        if (strategy == STRATEGIE_FABIEN_MICHEL) strategieFabienMichel();
        else strategieIA54();   
    }
    

    private Turtle[] getBestPrey()
    {
        double dist_poursuite = 3;  //La distance proie-predateur pour lequel une proie est consideree comme poursuivie 
        double dist_x, dist_y, dist;
	int PredatorsChasing = 0, PredatorsChasingMax = Integer.MIN_VALUE;
	Turtle[]PreyExamined, PredatorExamined, BestPrey=null;
	
	for (int i=0; i<preyList.size(); i++)
	{
            //Teste toutes les proies
	    PreyExamined = (Turtle[]) preyList.get(i); 
	    
	    for (int j=0; j<predatorList.size(); j++)
	    {
                //Teste tous les predateurs environnants
		PredatorExamined = (Turtle[]) predatorList.get(j); 
	        
		dist_x = PreyExamined[0].getXVecteur(PredatorExamined[0].xcor(), PredatorExamined[0].ycor());
		dist_y = PreyExamined[0].getYVecteur(PredatorExamined[0].xcor(), PredatorExamined[0].ycor());
		
		//dist_x = getXVecteur(myTurtle[0].xcor(), myTurtle[0].ycor());
		//dist_y = getYVecteur(myTurtle[0].xcor(), myTurtle[0].ycor());
		
                dist = Math.sqrt(dist_x*dist_x + dist_y*dist_y);
		
		if (dist < dist_poursuite)
		    PredatorsChasing++;
	    }
	    
	    //Prend la proie examinee comme cible si celle-ci a plus de predateurs la poursuivant
	    //que les proies precedentes
	    if (PredatorsChasing > PredatorsChasingMax)
	    {
	        PredatorsChasingMax = PredatorsChasing;
		BestPrey = PreyExamined;
	    }

            PredatorsChasing = 0;  //Remise a zero du nombre de predateurs poursuivants
	}
	
	return BestPrey;
      
    }
    
    
    private void strategieFabienMichel()
    {
        //Dans le live() de Fabien Michel
      	setHeading(towardsAPreyFabienMichel(visionRadius));
        
        //Dans le move() du Fabien Michel
        if (countTurtlesAt(dx(),dy())>0)
              turnRight(50);
        if (countTurtlesAt(dx(),dy())>0)
              turnLeft(100);
        if (countTurtlesAt(dx(),dy())==0)
              fd(1);
        else turnRight(50);
    }
    
    double towardsAPreyFabienMichel(int radius)
    {
            for(int i=-radius;i<=radius;i++)
                    for(int j=-radius;j<=radius;j++)
                            if (! (i==0 && j==0) )
                            {
                                    Turtle[] tur = turtlesAt(i,j);
                                    if (tur != null && tur.length>0 && tur[0].getType()==PREY_TYPE) //instead of "instanceof". So prey can be another java class
                                            return towards(tur[0].xcor(),tur[0].ycor());
                            }
            return Math.random()*180;
    }    
    
    
   private Vector calculeVecteurAttraction(ArrayList TurtleList)
   {
      Vector vect_retour = new Vector();	
      Turtle[] myTurtleNearest = null;
      
      if (TurtleList == predatorList)
      {
        //Cherche le predateur la plus proche
        myTurtleNearest = getNearestTurtleInList(predatorList);
      } else {
	//Cherche la meilleure proie
        myTurtleNearest = getBestPrey();
      }
      
      vect_retour = vectorBetweenTurtles(myTurtleNearest);
      
      return vect_retour;
   }
    
    
    
    private void strategieIA54()
    {
        Vector vecteurAttraction = new Vector();

        //Cherche les tortues dans le rayon de vision du predateur
	searchTurtle();
	turnsPlayingBehavior++;
      
        //Calcule les vecteurs d'attraction des predateurs
	if (behavior == BEHAVIOR_WANDERING)
        {
            //Le predateur se deplace dans la nature
            if (preyList.size() != 0)
	    {
                //Si des proies sont dans le rayon de vision, elles sont chassees en priorite
                vecteurAttraction = calculeVecteurAttraction(preyList);

                //On place le predateur en mode chasse
		setBehavior(BEHAVIOR_CHASING);
            }
            
            else
            {
                //Teste s'il n'y a pas des predateurs environnants pour former un groupe
                if (predatorList.size() != 0 && getRegroupement() && predatorList.size() >= 2)
                {
                    //Le vecteur attraction calcule dirigera le predateur vers le predateur le plus proche de lui
                    vecteurAttraction = calculeVecteurAttraction(predatorList);

                    //On place le predateur en mode groupement
                    setBehavior(BEHAVIOR_GROUPING);
                }

            }
        }
        else if (behavior == BEHAVIOR_CHASING)
        {
            //La proie est pourchassee
            if (preyList.size() != 0)
	    {
                //Si des proies sont dans le rayon de vision, elles ont la priorite
                vecteurAttraction = calculeVecteurAttraction(preyList);

                //Si le predateur chasse depuis trop longtemps en trop petit nombre ou si la troupe fait moins de 2,
                //il relache sa proie pour ne pas etre obnubile
		if(turnsPlayingBehavior > nbTurnsChasingMax && predatorList.size() < 2 
                    && (strategy==STRATEGIE_ATTIRANCE_TIMEOUT_DIR_ALEATOIRE || strategy==STRATEGIE_ATTIRANCE_TIMEOUT_DIR_FIXE))
                {
                    setBehavior(BEHAVIOR_FORCE_FLEEING);
                }
            }
            else
            {
                //Il n'y a pas eu de proies de retrouvees : on part en mode balade (wandering)
		setBehavior(BEHAVIOR_WANDERING);
            }
          
        }
        else if (behavior == BEHAVIOR_GROUPING)
        {
            //Le predateur se deplace dans la nature
            if (preyList.size() != 0)
	    {
                //Si des proies sont dans le rayon de vision, elles sont chassees en priorite
                vecteurAttraction = calculeVecteurAttraction(preyList);

                //On place le predateur en mode chasse
		setBehavior(BEHAVIOR_CHASING);
            }
            else if (predatorList.size() != 0 && getRegroupement() && predatorList.size() <= getNbParGroupe())
            {
            //Teste s'il n'y a pas des predateurs environnants pour former un groupe
                vecteurAttraction = calculeVecteurAttraction(predatorList);

                if(turnsPlayingBehavior > nbTurnsGroupingMax)
                {
                    //Le predator s'est assez eloigne : on arrete la fuite forcee
                    setBehavior(BEHAVIOR_WANDERING);
                }
            }
            else
            {
                //Aucun predateur n'a ete trouve : on repasse en mode exploration
                setBehavior(BEHAVIOR_WANDERING);
            }
            
        }
        else if (behavior == BEHAVIOR_FORCE_FLEEING)
        {
            if(turnsPlayingBehavior > nbTurnsFleeingMax)
            {
                //Le predator s'est assez eloigne : on arrete la fuite forcee
                setBehavior(BEHAVIOR_WANDERING);
            }
        }


        
        //Positionne le predateur en fonction des calculs precedents
        if (behavior == BEHAVIOR_WANDERING) 
        {
            //Mode promenade : se balade au hasard 
            if (Math.random() < 0.5) 
                turnRight(Math.random()*1); 
            else  
                turnLeft(Math.random()*1);
        }
	else if (behavior == BEHAVIOR_FORCE_FLEEING)
        {
            setHeading(absoluteDirection);
        }
        else
        {
            //Il y a soit des proies soit des predateurs
            double angle = getAngleVector(vecteurAttraction);
            setHeading(angle);

	    //Ajoute un zest de random() au deplacement pour permettre au prdateur de se placer a gauche ou a droite
	    //de la proie et ainsi la capturer
            if (Math.random() < 0.5) turnRight(Math.random()*25); 
            else turnLeft(Math.random()*25);
        }

        //Communique a l'interface graphique l'orientation de la tortue
	setOrientation((int)getAngle());  
      
        //Si le monde est torique
	if (!world.wrap && (xcor()<=1 || ycor()<=1 || xcor()>=world.x || ycor()>=world.y))
	{
	  changeDirection();
	  fd(1);
	} else 
	if (isTurtleAt() == true)  // Teste si il y a des Turtles dans l'emplacement cible
	{
	   //Oriente le predateur de 50 degres a gauche pour qu'il aille a son nord-ouest
	   turnLeft(50);
	   if (isTurtleAt() == true)
	   {
	     //Oriente le predateur de 100 degres a droite (soit a 50 degres a droite de sa position initiale)
	     //pour qu'il aille a son nord-est
	     turnRight(100);
	     if (isTurtleAt() == true)
	     {
		//On va tester maintenant toutes les positions possibles pendant 50 fois
		int i=0;
		//Recherche un emplacement libre jusqu'a en trouver un
                while (isTurtleAt() && i<50)
		    {
			setHeading(Math.random()*360);
			i++;
		    }
		if (i<50) fd(1);
	     } else fd(1);
	   } else fd(1);
	 } else fd(1);

    }

    
    void searchTurtle()
    {

        double norme = 0.0;
    
        // purge des tableaux de turtle
	preyList.clear();
	predatorList.clear();
    
	for (int i=-visionRadius ; i<=visionRadius ; i++)
	    {
		for (int j=-visionRadius ; j<=visionRadius ; j++)
		    {
			if (! (i==0 && j==0) )
			    {
				norme = Math.sqrt(j*j + i*i);
				if (norme <= visionRadius)
				    {
					Turtle[] tur = turtlesAt(i,j);

					if ((tur != null) && (tur.length>0) && (tur[0].getType() == Turtle.PREY_TYPE))
					    {
						preyList.add(tur);
					    }
    		
					if ((tur != null) && (tur.length>0) && (tur[0].getType() == Turtle.PREDATOR_TYPE))
					    {
						predatorList.add(tur);
					    }

				    }
			    }
		    }
	    }
    }
    
}
