/*
* Prey.java -TurtleKit - A 'star logo' in MadKit
* Copyright (C) 2000-2002 Fabien Michel
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

import madkit.kernel.Message;
import java.awt.Color;
import java.util.*;
import java.awt.Image;
import com.sun.image.codec.jpeg.*;
import java.io.*;

/** A Prey

  @author Fabien MICHEL
  @version 1.1 17/10/2000 */

public class Prey extends Turtle 
{    
    public static final int DEFAULT_VISION = 3;
    public static final int DEFAULT_SPEED  = 90;
    public static final int DEFAULT_GROUPING_NB = 4;
    public static final boolean DEFAULT_GROUPING = false;

    public static final int STRATEGIE_FABIEN_MICHEL=0, STRATEGIE_FUITE_PURE=1; 
    public static final int BEHAVIOR_WANDERING=1, BEHAVIOR_FLEEING=2, BEHAVIOR_GROUPING=3; 
    public int behavior = BEHAVIOR_WANDERING;
    
    private int preyNbMovesOrig, preyCurrentStep;
    private float preySpeedVal, preyNbMoves;
    private Image img;

    ArrayList preyList;
    ArrayList predatorList;
    

    public Prey(TurtleEnvironment w, int xOrig, int yOrig)
    {
	super(w, xOrig, yOrig, Turtle.PREY_TYPE);

	preyNbMoves       = DEFAULT_SPEED;
	preyNbMovesOrig   = DEFAULT_SPEED;
	regroupement      = DEFAULT_GROUPING;
	nbParGroupe       = DEFAULT_GROUPING_NB;
	preyCurrentStep   = 100;
	preySpeedVal      = (float) ((float) DEFAULT_SPEED / 100);
	visionRadius      = DEFAULT_VISION;
	preyList          = new ArrayList();
	predatorList      = new ArrayList();
        strategy          = STRATEGIE_FABIEN_MICHEL;
    }

    public void activate()
    {
	if(!isGroup("hunt"))
	    {
		createGroup(true, "hunt", null, null);
	    }
	
    	requestRole("hunt", "prey", null);
    	requestRole("hunt", "turtle", null);

	randomHeading();

	if (countTurtlesHere()>0)
	    fd(1);
    }

    public void setSpeed(int val)
    {
	preyNbMoves       = val;
	preyNbMovesOrig   = val;
	preyCurrentStep   = 100;
	preySpeedVal      = (float) ((float) val / 100);
    }

    public void end()
    {
	System.out.println("[Prey] End");
	leaveGroup("hunt");
    }

    private float myabs(float a)
    {
	if(a < 0)
	    {
		return -a;
	    }

	return a;
    }
    
    //a behavior
    public boolean live()
    {
	if (catched())
	    {
		return false;
	    }
	
	if(preyCurrentStep == 1)
	{
		if(preyNbMoves != 0)
	        {
			move();
	        }

		preyCurrentStep = 100;
		preyNbMoves = preyNbMovesOrig;
	    }
	else
	    {
		    
		if( myabs(preySpeedVal - ((float)(preyNbMoves)/(float)(preyCurrentStep-1))) > 
		    myabs(preySpeedVal - ((float)(preyNbMoves-1)/(float)(preyCurrentStep-1))))
		    {
			preyNbMoves --;
			move();
		    }

		preyCurrentStep--;
	    }

	return true;
   }
    

   private void move()
   {
        if (strategy == STRATEGIE_FABIEN_MICHEL) strategieFabienMichel();
        else strategieFuite();
   }
 
 
   private Vector calculeVecteurRepulsion()
   {
      Vector vect_retour = new Vector();	
      Turtle[] myTurtleNearest = null;
      
      //Cherche le predateur le plus proche et ensuite calcule le vecteur entre les deux agents
      myTurtleNearest = getNearestTurtleInList(predatorList);
      vect_retour = vectorBetweenTurtles(myTurtleNearest);
      
      return vect_retour;
   }

   private Vector calculeVecteurAttraction()
   {
      Vector vect_retour = new Vector();	
      Turtle[] myTurtleNearest = null;
      
      //Cherche la proie la plus proche et ensuite calcule le vecteur entre ces deux agents
      myTurtleNearest = getNearestTurtleInList(preyList);
      vect_retour = vectorBetweenTurtles(myTurtleNearest);
      
      return vect_retour;
   }

   
   
   private void strategieFabienMichel()
   {
       //Les deux lignes suivantes sont dans la partie du live() de Fabien Michel
       turnRight(Math.random()*60);
       turnLeft(Math.random()*60);


       //  <<<<<<Copier/Coller de la strategie dans le move() de FAbien Michel
       for (int i = 0;i<4;i++)
           if (countTurtlesAt(dx(),dy())>0)
	   {
		  if (Math.random() > .5) turnRight(Math.random()*170);
		  else turnLeft(Math.random()*170);
	   }
       // avoid being two on the same patch  
       if (countTurtlesAt(dx(),dy())==0)
           fd(1);

       //  >>>>>>>Fin du copier/coller de la strategie dans le move() de Fabien Michel
   }
   
   private void strategieFuite()
   {
      //Recherche les proies et les prdateurs prsents dans le champ de vision
      searchTurtle();
	
      Vector vecteurAttraction = new Vector();
      Vector vecteurRepulsion = new Vector();

      if (behavior == BEHAVIOR_WANDERING || behavior == BEHAVIOR_GROUPING)
      {  
          //Si la proie se deplace, met en priorite l'accent sur la fuite en cas de vision d'un predateur
          if (predatorList.size() != 0)
          {
            //Calcul du vecteur de rpulsion
            vecteurRepulsion = calculeVecteurRepulsion();
	    
            behavior = BEHAVIOR_FLEEING;
          }
          else if (preyList.size() != 0 && getRegroupement() && preyList.size() < getNbParGroupe())
          {
              //Calcul du vecteur d'attraction
              vecteurAttraction = calculeVecteurAttraction();
            
              behavior = BEHAVIOR_GROUPING;
          }
          else behavior=BEHAVIOR_WANDERING;
      }
      else if (behavior == BEHAVIOR_FLEEING)
      {
          if (predatorList.size() != 0)
          {
            //Calcul du vecteur de rpulsion
            vecteurRepulsion = calculeVecteurRepulsion();
          }
          else behavior=BEHAVIOR_WANDERING;
      }

      
      //Positionne la proie
      if (behavior == BEHAVIOR_WANDERING) 
      {
          //Mode promenade : la proie se deplace au hasard
          if (Math.random()<0.5) turnRight(Math.random()*50); else turnLeft(Math.random()*50);
      }
      else if (behavior == BEHAVIOR_GROUPING) 
      {
          //Mode regroupement : la proie est attiree par la plus proche proie
          double angle = getAngleVector(vecteurAttraction);
	  setHeading(angle);

          //Ajoute un petit caractere aleatoire au mouvement
          if (Math.random()<0.5) turnRight(Math.random()*10); else  turnLeft(Math.random()*10);
      }
      else 
      {
	  //Mode fuite face a un predateur
          //Calcul des angles
          double angle = getAngleVector(vecteurRepulsion);
	  setHeading(angle + 180);  //On ajoute le 180 pour inverser le sens et provoquer la fuite

          //Ajoute un petit caractere aleatoire au mouvement
          if (Math.random()<0.5) turnRight(Math.random()*20); else  turnLeft(Math.random()*20);
      }
      
      
      //Communique a l'interface graphique l'orientation de la tortue
      setOrientation((int)getAngle());
      

      // Teste si l'emplacement cible est vide de tortues et celle-ci deplace le cas echeant 
      if (isTurtleAt() == true)    //isTurtleAt() teste si il y a des tortues dans l'emplacement
      {
          int i=0;
          while (isTurtleAt() && i<50)
          {
            setHeading(Math.random()*360);  //Se positionne au hasard 
            i++;
          }
          if (i<50) fd(1);
      }
      else fd(1);
   }

   
    
    //Teste si la proie s'est fait attraper
    boolean catched()
    {
	int cpt=0;

	for (int i=-1 ; i<=1 ; i++)

	    for (int j=-1 ; j<=1 ; j++)

		if (! (i==0 && j==0) )
		    {
			Turtle[] tur = turtlesAt(i,j);

			if ((tur!= null) 
			    && (tur.length>0) 
			    && (tur[0].getType() == Turtle.PREDATOR_TYPE))
			    // instead of "instanceof". So predator
			    // can be another java class

			    cpt++;
		    }
	if (cpt>3) 
	{
	    return true;
	}
	return false;
    }
    
    // recherche les proies et les predateurs prsents dans le champ de vision de la proie
    void searchTurtle()
    {
        double norme = 0.0;
    
        // purge des tableaux de turtle
	preyList.clear();
	predatorList.clear();
    
	for (int i=-visionRadius ; i<=visionRadius ; i++)
	    for (int j=-visionRadius ; j<=visionRadius ; j++)
		if (! (i==0 && j==0) )
		    {
			norme = Math.sqrt(j*j + i*i);
			if (norme <= visionRadius)
			{
			    Turtle[] tur = turtlesAt(i,j);

			    if ((tur != null) && (tur.length>0)	&& (tur[0].getType() == Turtle.PREY_TYPE))
			    {
				preyList.add(tur);
			    }
    			    if ((tur != null) && (tur.length>0)	&& (tur[0].getType() == Turtle.PREDATOR_TYPE))
			    {
				predatorList.add(tur);
			    }

			 }
		    }
    }
    
}





