/*
* Turtle.java -TurtleKit - A 'star logo' in MadKit
* Copyright (C) 2000-2002 Fabien Michel
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

import madkit.kernel.*;
import java.awt.Color;
import java.awt.Image;
import java.lang.reflect.Method;
import java.util.*;
import java.util.Hashtable;
import java.util.Vector;


/** The Turtle class implements the Turtle commands which are used to
 * move set heading...
 *
 *  @author Fabien MICHEL
 *  @version 1.2 4/1/2000 
 */
public abstract class Turtle extends AbstractAgent implements ReferenceableAgent
{
    /** default direction values: setHeading(East) ~ setHeading(0)*/
    static public final int East=0,NorthEast=45,North=90,NorthWest=135;
    static public final int West=180,SouthWest=225,South=270,SouthEast=315;
    static public final int PREY_TYPE=1, PREDATOR_TYPE=2;
    
    /** Type of the Turtle, either Turtle.PREY_TYPE or
     * Turtle.PREDATOR_TYPE */
    private int type;
    private  double angle, x, y, angleCos = 1, angleSin = 0;

    protected TurtleEnvironment world;
    protected int visionRadius;
    protected boolean regroupement = true;
    protected int nbParGroupe = 8;
    protected int orientation = -1;        
    protected double xAttrac;
    protected double yAttrac;
    protected double xRepuls;
    protected double yRepuls;
    protected int strategy;
 
    Patch position;
    
    public Turtle(TurtleEnvironment w, int xOrig, int yOrig, int type)
    {
	world    = w;
	world.addTurtle(this, xOrig, yOrig);
	x        = xOrig;
	y        = yOrig;
	this.type = type;
    }

    public abstract boolean live();
    public abstract void    end();
    public abstract void    activate();

    public Image getImage()
    { 
	return null; 
    }

    protected void die()
    {
	world.removeTurtle(this);
    }

    public int getType()
    {
	return type;
    }

    public double getXAttrac()
    {
	return xAttrac;
    }

    public void setXAttrac(double x)
    {
	xAttrac = x;
    }

    public double getYAttrac()
    {
	return yAttrac;
    }

    public void setYAttrac(double y)
    {
	yAttrac = y;
    }

    public double getXRepuls()
    {
	return xRepuls;
    }

    public void setXRepuls(double x)
    {
	xRepuls = x;
    }

    public double getYRepuls()
    {
	return yRepuls;
    }

    public void setYRepuls(double y)
    {
	yRepuls = y;
    }

    public int getOrientation()
    {
	return orientation;
    }

    public void setOrientation(int or)
    {
	orientation = or;
    }
    
    public boolean getRegroupement()
    {
	return regroupement;
    }

    public void setRegroupement(boolean rg)
    {
	regroupement = rg;
    }    
    
    public double getAngle() 
    { 
	return angle;
    }

    public int getVisionRadius()
    {
	return visionRadius;
    }

    public void setVisionRadius(int vr)
    {
	visionRadius = vr;
    }

    public int getNbParGroupe()
    {
	return nbParGroupe;
    }

    public void setNbParGroupe(int nb)
    {
	nbParGroupe = nb;
    }

    
    final double normeX(double a)
    {
	if (world.wrap)	
	    {
		if (a>world.x)
		    return a%world.x;
		else
		    {
			if (a < 0) 
			    return a+world.x;
			else 
			    return a;
		    }
	    }
	else
	    {
		if (a>=(world.x-0.5))
		    return world.x-1;
		else
		    {
			if (a<0) 
			    return 0;
			else 
			    return a;
		    }
	    }
    }
    
    final double normeY(double a)
    {
	if (world.wrap)	
	    {
		if (a>world.y)
		    return a%world.y;
		else
		    {
			if (a < 0) 
			    return a+world.y;
			else 
			    return a;
		    }
	    }
	else
	    {
		if (a>=(world.y-0.5))
		    return world.y-1;
		else
		    {
			if (a<0) 
			    return 0;
			else 
			    return a;
		    }
	    }
    }

    final int normeX(int a)
    {
	if (world.wrap)
	    {	
		a %=world.x;

		if (a < 0) 
		    return a+world.x;
		else 
		    return a;
	    }
	else
	    {
		if (a>=world.x)
		    return world.x-1;
		else
		    if (a<0) 
			return 0;
		else 
		    return a;
	    }
    }
    
    final int normeY(int a)
    {
	if (world.wrap)
	    {	
		a %=world.y;
		if (a < 0) 
		    return a+world.y;
		else 
		    return a;
	  }
	else
	    {
		if (a>=world.y)
		    return world.y-1;
		else
		    {
			if (a<0) 
			    return 0;
			else 
			    return a;
		    }
	    }
    }
    
    public String toString()
    {
	return "turtle at "+xcor()+" "+ycor()+
	    " : heading="+angle;
    }
    
    /**return the current heading of the turtle*/
    public final double getHeading()
    {
	return angle;
    }

    /**set the turtle heading to the value of direction*/
    public final void setHeading(double direction)
    {
	angle = direction%360;
	if (angle < 0) 
	    angle+=360;

	angleSin=Math.sin( (Math.PI*angle)/180);
	angleCos=Math.cos( (Math.PI*angle)/180);
    }
    
    /**turtle move forward*/
    public final void fd(int nb)
    {
	for(int i = 0;i < nb;i++)
	    {
		x = normeX(x+angleCos);
		y = normeY(y+angleSin);
		world.moveTurtle(x,y,this);
	    }
    }

    /** turtle move backward*/
    public final void bk(int nb)
    {
	for(int i = 0;i < nb;i++)
	    {
		x = normeX(x-angleCos);
		y = normeY(y-angleSin);
		world.moveTurtle(x,y,this);
	    }
    }

    /** teleport the turtle to patch (a,b).
	Can be used as a jump primitive: MoveTo(xcor()+10,ycor())*/
    public final void moveTo(int a,int b)
    {
	x = normeX(a);
	y = normeY(b);
	world.moveTurtle(x,y,this);
    }
    
    /**teleport the turtle to the center patch*/ 
    public final void home()
    {
	x=world.x/2;
	y=world.y/2;
	world.moveTurtle(x,y,this);
    }
    
    public final void setX(double a)
    {
	x = normeX(a);
	
	world.moveTurtle(x,y,this);
    }

    public final void setY(double b)
    {
	y = normeY(b);
	
	world.moveTurtle(x,y,this);
    }

    public final void setXY(double a,double b)
    {
	x = normeX(a);
	y = normeY(b);
	
	world.moveTurtle(x,y,this);
    }

    /**return the "onscreen distance" between turtle the patch (a,b)*/
    final double distanceNowrap(double a,double b)
    {
	a = normeX(a);
	b = normeY(b);
	a-=x;
	b-=y;
	return Math.sqrt( a*a + b*b );
    }

    /**
     * Returns the distance from the patch (a,b). 
     * The "wrapped distance", when wrap mode is on, (around the edges
     * of the screen) if that distance is shorter than the "onscreen
     * distance."
     */
    public final double distance(double a,double b)
    {
	if (! world.wrap) 
	    return distanceNowrap(a,b);

	a = normeX(a);
	b = normeY(b);

	if (Math.abs(a-x) > world.x/2)
	    {
		if (a < x) 
		    a=a+world.x;
		else 
		    a-=world.x;
	    }

	if (Math.abs(b-y) > world.y/2)
	    {
		if (b < y) 
		    b=b+world.y;
		else 
		    b=b-world.y;
	    }
	
	a-=x;
	b-=y;

	return Math.sqrt( a*a + b*b );
    }

    final double towardsNowrap(double a,double b)
    {
	a = normeX(a);
	b = normeY(b);
	a-=x;
	b-=y;

	if (a == 0 && b == 0) 
	    return 0.0;

	if (b < 0)
	    {
		return 180*Math.asin(a / Math.sqrt(a*a+b*b))/Math.PI+270;
	    }
	else
	    {
		return 180*Math.acos(a / Math.sqrt(a*a+b*b))/Math.PI;
	    }
    }
    
    /**
     * returns direction to the patch (a,b).  If the "wrapped
     * distance", when wrap mode is on, (around the edges of the
     * screen) is shorter than the "onscreen distance," towards will
     * report the direction of the wrapped path, otherwise it while
     * will report the direction of the onscreen path
     */
    public final double towards(double a,double b)
    {
	if (! world.wrap) 
	    return towardsNowrap(a,b);

	if (distance(a,b) > distanceNowrap(a,b))
	    {
		return towardsNowrap(a,b);
	    }
	else 
	    {
		a = normeX(a);
		b = normeY(b);

		if (Math.abs(a-x) > world.x/2)
		    {
			if (a < x) 
			    a=a+world.x;
			else 
			    a-=world.x;
		    }
		if (Math.abs(b-y) > world.y/2)
		    {
			if (b < y) 
			    b=b+world.y;
			else 
			    b=b-world.y;
		    }

		a-=x;
		b-=y;

		if (a == 0 && b == 0) 
		    return 0.0;

		if (b < 0)
		    {
			return 180*Math.asin(a / Math.sqrt(a*a+b*b))/
			    Math.PI+270;
		    }
		else
		    {
			return 180*Math.acos(a / Math.sqrt(a*a+b*b))/Math.PI;
		    }
	    }
    }
    
    public final void randomHeading() 
    { 
	setHeading(Math.random()*360); 
    }
    
    public final int xcor()
    { 
	return ((int) Math.round(x))%world.x; 
    }

    public final int ycor()
    { 
	return ((int) Math.round(y))%world.y; 
    }
    
    /**return the x-increment if the turtle were to take one
       step forward in its current heading.*/
    public final int dx()
    {
	return (int) (Math.round(x+angleCos)-Math.round(x));
    }

    /**return the y-increment if the turtle were to take one
       step forward in its current heading.*/
    public final int dy()
    {
	return (int) (Math.round(y+angleSin)-Math.round(y));
    }
    
    public final void turnRight(double a)
    {
	angle-=a;
	setHeading(angle);
    }
    
    public final void turnLeft(double a)
    {
	angle+=a;
	setHeading(angle);
    }
    
    /**return other turtles on the current patch*/
    public final Turtle[] turtlesHere() 
    {
	return position.getOtherTurtles(this);
    }

    /**return turtles who are on the patch situated at (a,b) units away*/
    public final Turtle[] turtlesAt(int a,int b)
    {
	return world.turtlesAt(normeX(a+xcor()),normeY(b+ycor()));
    }
    
    public final int countTurtlesHere()
    {
	return position.turtlesHere.size();
    }

    /**return the number of turtles in the patch situated at (a,b) units away*/
    public final int countTurtlesAt(int a,int b)
    {
	return world.turtlesCountAt(normeX(a+xcor()),normeY(b+ycor()));
    }
    
    public final int getWorldWidth() 
    {
	return world.x;
    }

    public final int getWorldHeight()
    {
	return world.y;
    }

    //////////////////////////////////////////////
   // COmpte les tortues dans la case de destination
    public final boolean isTurtleAt()
    {
       int valeur;
       fd(1);
       Turtle[] tur = turtlesAt(0, 0);
       valeur = tur.length;
       bk(1);
       
       if (valeur > 1) return true;
       else return false;
       
    }

    public final double getXVecteur(double a, double b)
    {
 	 Vector vect_temp  = new Vector(2);
	 Double myDoubleTemp;
	 vect_temp = getCoordsVecteur(a, b);
         myDoubleTemp = (Double) vect_temp.get(0);
	 return myDoubleTemp.doubleValue();
    }

    public final double getYVecteur(double a, double b)
    {
 	 Vector vect_temp  = new Vector(2);
 	 Double myDoubleTemp;
	 vect_temp = getCoordsVecteur(a, b);
         myDoubleTemp = (Double) vect_temp.get(1);
	 return myDoubleTemp.doubleValue();
    }


    //Retourne le vecteur TortueCourante vers TortueTransmiseEnParametre
    public final Vector vectorBetweenTurtles(Turtle[] myTurtle)
    {
      Vector vect_retour = new Vector();
      
      vect_retour=getCoordsVecteur(myTurtle[0].xcor(), myTurtle[0].ycor());
      return vect_retour;
    }
    
 
    //Retourne un vecteur allant de la tortue en cours vers le point transmis en parametre : P(a,b)
    public final Vector getCoordsVecteur(double a,double b)
    {
	Vector vect_temp = new Vector();	

	if (! world.wrap) 
	{
          vect_temp.add(0, new Double(a-xcor()));
          vect_temp.add(1, new Double(b-ycor()));
	  return vect_temp;
	}

	a = normeX(a);
	b = normeY(b);

	if (Math.abs(a-x) > world.x/2)
	    if (a < x) a=a+world.x;
	    else a-=world.x;
	if (Math.abs(b-y) > world.y/2)
	    if (b < y) b=b+world.y;
	    else b=b-world.y;

	a-=x;
	b-=y;

        vect_temp.add(0, new Double(a));
        vect_temp.add(1, new Double(b));


	return vect_temp;
    }

    
    //Retourne la tortue la plus proche parmi celles de la liste
    public Turtle[] getNearestTurtleInList(ArrayList TurtleList)
    {
        double dist_x, dist_y, dist, dist_min=Integer.MAX_VALUE;
	Turtle[] myTurtle;
	Turtle[] myTurtleNearest = null;
        
        for (int i = 0; i < TurtleList.size(); i++)
        {
            //Cherche la proie la plus proche
            myTurtle = (Turtle[])TurtleList.get(i);

            dist_x = getXVecteur(myTurtle[0].xcor(), myTurtle[0].ycor());
            dist_y = getYVecteur(myTurtle[0].xcor(), myTurtle[0].ycor());

            dist = Math.sqrt(dist_x*dist_x + dist_y*dist_y);
            if (dist_min > dist)
            {
                myTurtleNearest = myTurtle;
                dist_min = dist;
            }
        }
        
        return myTurtleNearest;
    }


    public void setStrategy(int s)
    {
       strategy = s; 
    }

    
    //Retourne l'angle forme par le vecteur transmis en parametre
    public double getAngleVector(Vector vect)
    {
      double angle_retour;
      double x=((Double)vect.get(0)).doubleValue(), 
             y=((Double)vect.get(1)).doubleValue();
      
      if (x == 0)
      {
        if (y > 0)
          angle_retour = 90;
	else
	  angle_retour = 270;
      }
      else
      {
	if (x < 0) 
          angle_retour = ((180*Math.atan(y / x))/Math.PI+180);
	else 
	  angle_retour = ((180*Math.atan(y / x))/Math.PI); 
      }
      return angle_retour;
    }
    

}
