/*
* TurtleEnvironment.java -TurtleKit - A 'star logo' in MadKit
* Copyright (C) 2000-2002 Fabien Michel
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
import java.lang.reflect.*;
import java.util.*;
import java.awt.Color;
import madkit.kernel.*;

/**
 *
 * This Agent is the one who creates turtles,patches and who cares about
 * managing them all (creation, death...)
 *
 *  @author Fabien MICHEL
 *  @version 3.0 09/10/2001  
 */
public class TurtleEnvironment 
    extends AbstractAgent implements ReferenceableAgent
{
    /** The grid of the world */
    public Patch grid[][] = null;

    /** Width of the world */
    int x;

    /** Height of the world */
    int y;

    /** Number of turtles */
    int turtlesCount=-1;

    /** The group of the TurtleEnvironment agent */
    String simulationGroup;

    /** World is in wrapping mode */
    boolean wrap = true;

    /**
     * TurtleEnvironnement constructor. 
     *
     * @param width  Width of the world
     * @param height Height of the world
     * @param group  The group in which the TurleEnvironnement agent
     *               should be resgistered
     */
    public TurtleEnvironment (int width, int height, String group)
    {
	x = width;
	y = height;
 	grid = new Patch[x][y];
	initGrid();
 	this.simulationGroup = group;
    }

    public int getWorldWidth()
    {
	return x;
    }

    public int getWorldHeight()
    {
	return y;
    }

    public void setWrappingMode(boolean wrap)
    {
	this.wrap = wrap;
    }

    final void initGrid()
    {
	for (int i=0; i < x; i++)
	    for (int j=0; j < y; j++)
		    grid[i][j] = new Patch();
    }
    
    
    final void addTurtle(Turtle t, int a, int b)
    {
	grid[a % x][b % y].addAgent(t);
    }
    
    final void removeTurtle(Turtle t)
    {
	(t.position).removeAgent(t);
	killAgent(t);
    }

    final Turtle[] turtlesAt(int u,int z)
    {
	return grid[u][z].getTurtles();
    }
    
    final int turtlesCountAt(int u, int v)
    {
	return grid[u][v].size();
    }
    
    final void moveTurtle(double a,double b,Turtle t)
    {
	int u = ((int) Math.round(a))%x;
	int v = ((int) Math.round(b))%y;

	if (grid[u][v] != t.position)
	    {
		(t.position).removeAgent(t);
		grid[u][v].addAgent(t);
	    }
    }

    final public void activate()
    {
	// joinGroup(simulationGroup);
	// requestRole(simulationGroup, "world");

	if(! isGroup(simulationGroup))
	    {
		createGroup(true, simulationGroup, null, null);
	    }
	
	requestRole(simulationGroup, "world", null);
    }

    final public void end()
    {
	System.out.println("[TurtleEnvironment] End");
	leaveGroup(simulationGroup);
    }
}
