/*
* TurtleScheduler.java -TurtleKit - A 'star logo' in MadKit
* Copyright (C) 2000-2002 Fabien Michel
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

import madkit.kernel.Message;
import madkit.lib.simulation.TurboMethodActivator;

/** The TurtleKit scheduler

  @author Fabien MICHEL
  @version 2.1 10/04/2002 

*/

public class TurtleScheduler extends madkit.kernel.Scheduler
{
    public static final int RUNNING_MODE=1, STEP_BY_STEP_MODE=2, PAUSED_MODE=3, READY_MODE=4;
    String group;
    TurboMethodActivator viewerDoIt, counterDoIt;
    TurtleActivator turtleDoIt;
    int delay = 10;
    int mode = READY_MODE;
    int steps = 0;
    

    public void setMode(int mode)
    {
	this.mode = mode;
    }

    public int getMode()
    {
	return mode;
    }

    public void giveOneMoreStep()
    {
	steps++;
    }

    public void setSimulationSpeed(int speed)
    {
	delay = (int) ((100 / (double) speed) * 10);
    }

    public TurtleScheduler(String group)
    {
	this.group=group;
    }

    final public void activate()
    {
	//	joinGroup(group);
	//	requestRole(group, "scheduler");

	if(!isGroup(group))
	    {
		createGroup(true, group, null, null);
	    }
	
    	requestRole(group, "scheduler", null);

	/* We cannot use a TurboMethodActivator to execute() the
	 * turtles, because they can be deleted during execution. */
	turtleDoIt = new TurtleActivator(group);
	addActivator(turtleDoIt);

	/* The viewerDoIt TurboMethodActivator will allow us to call
	 * the display() method of the agent with the role
	 * "viewer". In our application, the only agent with this role
	 * is implemented in the Viewer class, and allows to display a
	 * graphical representation of the world */
	viewerDoIt = new TurboMethodActivator("display",group,"viewer");
	addActivator(viewerDoIt);

	counterDoIt = new TurboMethodActivator("display",group,"counter");
	addActivator(counterDoIt);	
    }
    
    final public void live()
    {
	while(true)
	    {
		exitImmediatlyOnKill();
		if (delay == 0)
		    Thread.yield();
		else
		    pause(delay);

		viewerDoIt.execute();

		if(mode == RUNNING_MODE)
		    {
			turtleDoIt.execute();
			counterDoIt.execute();
		    }
		else if(mode == STEP_BY_STEP_MODE)
		    {
			if(steps > 0)
			    {
				turtleDoIt.execute();
				counterDoIt.execute();
				steps--;
			    }
		    }
	    }
    }
    
    public void end()
    {
	System.out.println("[TurtleScheduler] End");
	removeAllActivators();
	leaveGroup(group);
    }
}
