/*
* Viewer.java -TurtleKit - A 'star logo' in MadKit
* Copyright (C) 2000-2002 Fabien Michel
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

import madkit.kernel.*;
import java.awt.Color;

/** Viewer is the simulation default world displayer agent (an
    specialized Observer, it can be extended to redefine the
    default representation of patches and turtles (a square fill
    with the color return by the getColor() method of them).

    @author Fabien MICHEL
    @version 2.0 22/08/2000 */

public class Viewer extends Watcher implements ReferenceableAgent
{
    TurtleProbe allTurtles;
    EnvironnementPanel envPanel;
    String group;
    boolean graphicInfo, withImages;
    
    public Viewer(String group, EnvironnementPanel p, boolean withImages)
    {
	super();
	envPanel = p;
	this.group = group;
	this.withImages = withImages;
    }

    public void activate()  
    {
	// joinGroup(group);
	// requestRole(group, "viewer");
	
	if(!isGroup(group))
	    {
		createGroup(true, group, null, null);
	    }
	
    	requestRole(group, "viewer", null);

    	allTurtles = new TurtleProbe(group, "turtle");
    	addProbe(allTurtles);
    }

    public void end()
    {
	System.out.println("[Viewer] End");
	leaveGroup(group);
    }

    public void setGraphicInfoMode(boolean graphicInfo)
    {
	this.graphicInfo = graphicInfo;
    }

    private void displayTurtle(Turtle t)
    {
	envPanel.paintTurtle(t.xcor(), t.ycor(), t.getType());

	if(graphicInfo)
	    {
		envPanel.paintInfo(t.xcor(), t.ycor(), t.getType(), 
				   t.getOrientation(),
				   t.getVisionRadius(),
				   t.getXAttrac(),
				   t.getYAttrac(),
				   t.getXRepuls(),
				   t.getYRepuls());
	    }
    }

    public void display()
    {

	Turtle[] turtles = allTurtles.getTurtles();

	envPanel.reset();
	
	for(int i=0;i<turtles.length;i++)
	    {
		Turtle t = turtles[i];

		if (t != null)
		    {
			displayTurtle(t);
		    }
	    }

	envPanel.repaint();
    }
}

